@@ -16,15 +16,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_tar.c,v 1.20 2001/10/28 06:25:58 momjian Exp $
- *
- * Modifications - 28-Jun-2000 - pjw@rhyme.com.au
- *
- *	Initial version.
- *
- * Modifications - 04-Jan-2001 - pjw@rhyme.com.au
- *
- *	  - Check results of IO routines more carefully.
+ *		$Header: /cvsroot/pgsql/src/bin/pg_dump/pg_backup_tar.c,v 1.32.2.2 2003/02/01 19:29:26 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -36,16 +28,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include <unistd.h>
 
 static void _ArchiveEntry(ArchiveHandle *AH, TocEntry *te);
 static void _StartData(ArchiveHandle *AH, TocEntry *te);
-static int	_WriteData(ArchiveHandle *AH, const void *data, int dLen);
+static size_t _WriteData(ArchiveHandle *AH, const void *data, size_t dLen);
 static void _EndData(ArchiveHandle *AH, TocEntry *te);
 static int	_WriteByte(ArchiveHandle *AH, const int i);
 static int	_ReadByte(ArchiveHandle *);
-static int	_WriteBuf(ArchiveHandle *AH, const void *buf, int len);
-static int	_ReadBuf(ArchiveHandle *AH, void *buf, int len);
+static size_t _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len);
+static size_t _ReadBuf(ArchiveHandle *AH, void *buf, size_t len);
 static void _CloseArchive(ArchiveHandle *AH);
 static void _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt);
 static void _WriteExtraToc(ArchiveHandle *AH, TocEntry *te);
@@ -76,19 +69,30 @@ typedef struct
 	FILE	   *tmpFH;
 	char	   *targetFile;
 	char		mode;
-	int			pos;
-	int			fileLen;
+	off_t		pos;
+	off_t		fileLen;
 	ArchiveHandle *AH;
 } TAR_MEMBER;
 
+/*
+ * Maximum file size for a tar member: The limit inherent in the
+ * format is 2^33-1 bytes (nearly 8 GB).  But we don't want to exceed
+ * what we can represent by an off_t.
+ */
+#ifdef INT64_IS_BUSTED
+#define MAX_TAR_MEMBER_FILELEN INT_MAX
+#else
+#define MAX_TAR_MEMBER_FILELEN (((int64) 1 << Min(33, sizeof(off_t)*8 - 1)) - 1)
+#endif
+
 typedef struct
 {
 	int			hasSeek;
-	int			filePos;
+	off_t		filePos;
 	TAR_MEMBER *blobToc;
 	FILE	   *tarFH;
-	int			tarFHpos;
-	int			tarNextMember;
+	off_t		tarFHpos;
+	off_t		tarNextMember;
 	TAR_MEMBER *FH;
 	int			isSpecialScript;
 	TAR_MEMBER *scriptTH;
@@ -108,20 +112,20 @@ static TAR_MEMBER *tarOpen(ArchiveHandle *AH, const char *filename, char mode);
 static void tarClose(ArchiveHandle *AH, TAR_MEMBER *TH);
 
 #ifdef __NOT_USED__
-static char *tarGets(char *buf, int len, TAR_MEMBER *th);
+static char *tarGets(char *buf, size_t len, TAR_MEMBER *th);
 #endif
 static int	tarPrintf(ArchiveHandle *AH, TAR_MEMBER *th, const char *fmt,...);
 
 static void _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th);
 static int	_tarChecksum(char *th);
 static TAR_MEMBER *_tarPositionTo(ArchiveHandle *AH, const char *filename);
-static int	tarRead(void *buf, int len, TAR_MEMBER *th);
-static int	tarWrite(const void *buf, int len, TAR_MEMBER *th);
+static size_t tarRead(void *buf, size_t len, TAR_MEMBER *th);
+static size_t tarWrite(const void *buf, size_t len, TAR_MEMBER *th);
 static void _tarWriteHeader(TAR_MEMBER *th);
 static int	_tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th);
-static int	_tarReadRaw(ArchiveHandle *AH, void *buf, int len, TAR_MEMBER *th, FILE *fh);
+static size_t _tarReadRaw(ArchiveHandle *AH, void *buf, size_t len, TAR_MEMBER *th, FILE *fh);
 
-static int	_scriptOut(ArchiveHandle *AH, const void *buf, int len);
+static size_t _scriptOut(ArchiveHandle *AH, const void *buf, size_t len);
 
 /*
  *	Initializer
@@ -158,6 +162,12 @@ InitArchiveFmt_Tar(ArchiveHandle *AH)
 	AH->formatData = (void *) ctx;
 	ctx->filePos = 0;
 
+	/* Initialize LO buffering */
+	AH->lo_buf_size = LOBBUFSIZE;
+	AH->lo_buf = (void *) malloc(LOBBUFSIZE);
+	if (AH->lo_buf == NULL)
+		die_horribly(AH, modulename, "out of memory\n");
+
 	/*
 	 * Now open the TOC file
 	 */
@@ -181,7 +191,7 @@ InitArchiveFmt_Tar(ArchiveHandle *AH)
 		 */
 		/* setvbuf(ctx->tarFH, NULL, _IONBF, 0); */
 
-		ctx->hasSeek = (fseek(ctx->tarFH, 0, SEEK_CUR) == 0);
+		ctx->hasSeek = checkSeek(ctx->tarFH);
 
 		if (AH->compression < 0 || AH->compression > 9)
 			AH->compression = Z_DEFAULT_COMPRESSION;
@@ -218,7 +228,7 @@ InitArchiveFmt_Tar(ArchiveHandle *AH)
 
 		ctx->tarFHpos = 0;
 
-		ctx->hasSeek = (fseek(ctx->tarFH, 0, SEEK_CUR) == 0);
+		ctx->hasSeek = checkSeek(ctx->tarFH);
 
 		/*
 		 * Forcibly unmark the header as read since we use the lookahead
@@ -231,7 +241,6 @@ InitArchiveFmt_Tar(ArchiveHandle *AH)
 		ReadToc(AH);
 		tarClose(AH, ctx->FH);	/* Nothing else in the file... */
 	}
-
 }
 
 /*
@@ -345,7 +354,6 @@ tarOpen(ArchiveHandle *AH, const char *filename, char mode)
 		/* tm->zFH = gzdopen(dup(fileno(ctx->tarFH)), "rb"); */
 
 #else
-
 		tm->nFH = ctx->tarFH;
 #endif
 
@@ -415,10 +423,10 @@ tarClose(ArchiveHandle *AH, TAR_MEMBER *th)
 
 #ifdef __NOT_USED__
 static char *
-tarGets(char *buf, int len, TAR_MEMBER *th)
+tarGets(char *buf, size_t len, TAR_MEMBER *th)
 {
 	char	   *s;
-	int			cnt = 0;
+	size_t		cnt = 0;
 	char		c = ' ';
 	int			eof = 0;
 
@@ -458,13 +466,13 @@ tarGets(char *buf, int len, TAR_MEMBER *th)
  * Just read bytes from the archive. This is the low level read routine
  * that is used for ALL reads on a tar file.
  */
-static int
-_tarReadRaw(ArchiveHandle *AH, void *buf, int len, TAR_MEMBER *th, FILE *fh)
+static size_t
+_tarReadRaw(ArchiveHandle *AH, void *buf, size_t len, TAR_MEMBER *th, FILE *fh)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			avail;
-	int			used = 0;
-	int			res = 0;
+	size_t		avail;
+	size_t		used = 0;
+	size_t		res = 0;
 
 	avail = AH->lookaheadLen - AH->lookaheadPos;
 	if (avail > 0)
@@ -509,10 +517,10 @@ _tarReadRaw(ArchiveHandle *AH, void *buf, int len, TAR_MEMBER *th, FILE *fh)
 	return (res + used);
 }
 
-static int
-tarRead(void *buf, int len, TAR_MEMBER *th)
+static size_t
+tarRead(void *buf, size_t len, TAR_MEMBER *th)
 {
-	int			res;
+	size_t		res;
 
 	if (th->pos + len > th->fileLen)
 		len = th->fileLen - th->pos;
@@ -527,10 +535,10 @@ tarRead(void *buf, int len, TAR_MEMBER *th)
 	return res;
 }
 
-static int
-tarWrite(const void *buf, int len, TAR_MEMBER *th)
+static size_t
+tarWrite(const void *buf, size_t len, TAR_MEMBER *th)
 {
-	int			res;
+	size_t		res;
 
 	if (th->zFH != 0)
 		res = GZWRITE((void *) buf, 1, len, th->zFH);
@@ -539,14 +547,15 @@ tarWrite(const void *buf, int len, TAR_MEMBER *th)
 
 	if (res != len)
 		die_horribly(th->AH, modulename,
-					 "could not write to tar member (wrote %d, attempted %d)\n", res, len);
+			"could not write to tar member (wrote %lu, attempted %lu)\n",
+					 (unsigned long) res, (unsigned long) len);
 
 	th->pos += res;
 	return res;
 }
 
-static int
-_WriteData(ArchiveHandle *AH, const void *data, int dLen)
+static size_t
+_WriteData(ArchiveHandle *AH, const void *data, size_t dLen)
 {
 	lclTocEntry *tctx = (lclTocEntry *) AH->currToc->formatData;
 
@@ -573,7 +582,7 @@ _PrintFileData(ArchiveHandle *AH, char *filename, RestoreOptions *ropt)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 	char		buf[4096];
-	int			cnt;
+	size_t		cnt;
 	TAR_MEMBER *th;
 
 	if (!filename)
@@ -601,7 +610,7 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 	lclContext *ctx = (lclContext *) AH->formatData;
 	lclTocEntry *tctx = (lclTocEntry *) te->formatData;
 	char	   *tmpCopy;
-	int			i,
+	size_t		i,
 				pos1,
 				pos2;
 
@@ -637,7 +646,7 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 		if (tmpCopy[pos1] == '"')
 			pos1 += 2;
 
-		pos1 += strlen(te->name);
+		pos1 += strlen(te->tag);
 
 		for (pos2 = pos1; pos2 < strlen(tmpCopy); pos2++)
 			if (strncmp(&tmpCopy[pos2], "from stdin", 10) == 0)
@@ -645,8 +654,8 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 
 		if (pos2 >= strlen(tmpCopy))
 			die_horribly(AH, modulename,
-						 "bad COPY statement - could not find \"from stdin\" in string \"%s\" starting at position %d\n",
-						 tmpCopy, pos1);
+						 "bad COPY statement - could not find \"from stdin\" in string \"%s\" starting at position %lu\n",
+						 tmpCopy, (unsigned long) pos1);
 
 		ahwrite(tmpCopy, 1, pos2, AH);	/* 'copy "table" [with oids]' */
 		ahprintf(AH, " from '$$PATH$$/%s' %s", tctx->filename, &tmpCopy[pos2 + 10]);
@@ -660,41 +669,13 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 		_PrintFileData(AH, tctx->filename, ropt);
 }
 
-/* static void _getBlobTocEntry(ArchiveHandle* AH, int *oid, char fname[K_STD_BUF_SIZE])
- * {
- *	lclContext*		ctx = (lclContext*)AH->formatData;
- *	char			blobTe[K_STD_BUF_SIZE];
- *	int				fpos;
- *	int				eos;
- *
- *	if (tarGets(&blobTe[0], K_STD_BUF_SIZE - 1, ctx->blobToc) != NULL)
- *	{
- *		*oid = atoi(blobTe);
- *
- *		fpos = strcspn(blobTe, " ");
- *
- *		strncpy(fname, &blobTe[fpos+1], K_STD_BUF_SIZE - 1);
- *
- *		eos = strlen(fname)-1;
- *
- *		if (fname[eos] == '\n')
- *			fname[eos] = '\0';
- *
- *	} else {
- *
- *		*oid = 0;
- *		fname[0] = '\0';
- *	}
- *}
- */
-
 static void
 _LoadBlobs(ArchiveHandle *AH, RestoreOptions *ropt)
 {
 	Oid			oid;
 	lclContext *ctx = (lclContext *) AH->formatData;
 	TAR_MEMBER *th;
-	int			cnt;
+	size_t		cnt;
 	char		buf[4096];
 
 	StartRestoreBlobs(AH);
@@ -704,29 +685,29 @@ _LoadBlobs(ArchiveHandle *AH, RestoreOptions *ropt)
 	{
 		ctx->FH = th;
 
-		oid = (Oid) strtoul(&th->targetFile[5], NULL, 10);
-
-		if (strncmp(th->targetFile, "blob_", 5) == 0 && oid != 0)
+		if (strncmp(th->targetFile, "blob_", 5) == 0)
 		{
-			ahlog(AH, 1, "restoring large object OID %u\n", oid);
-
-			StartRestoreBlob(AH, oid);
-
-			while ((cnt = tarRead(buf, 4095, th)) > 0)
+			oid = atooid(&th->targetFile[5]);
+			if (oid != 0)
 			{
-				buf[cnt] = '\0';
-				ahwrite(buf, 1, cnt, AH);
+				ahlog(AH, 1, "restoring large object OID %u\n", oid);
+
+				StartRestoreBlob(AH, oid);
+
+				while ((cnt = tarRead(buf, 4095, th)) > 0)
+				{
+					buf[cnt] = '\0';
+					ahwrite(buf, 1, cnt, AH);
+				}
+				EndRestoreBlob(AH, oid);
 			}
-			EndRestoreBlob(AH, oid);
 		}
 
 		tarClose(AH, th);
 
 		th = tarOpen(AH, NULL, 'r');
 	}
-
 	EndRestoreBlobs(AH);
-
 }
 
 
@@ -756,22 +737,22 @@ _ReadByte(ArchiveHandle *AH)
 	return c;
 }
 
-static int
-_WriteBuf(ArchiveHandle *AH, const void *buf, int len)
+static size_t
+_WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
+	size_t		res;
 
 	res = tarWrite((void *) buf, len, ctx->FH);
 	ctx->filePos += res;
 	return res;
 }
 
-static int
-_ReadBuf(ArchiveHandle *AH, void *buf, int len)
+static size_t
+_ReadBuf(ArchiveHandle *AH, void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
+	size_t		res;
 
 	res = tarRead(buf, len, ctx->FH);
 	ctx->filePos += res;
@@ -829,7 +810,7 @@ _CloseArchive(ArchiveHandle *AH)
 		ropt = NewRestoreOptions();
 		ropt->dropSchema = 1;
 		ropt->compression = 0;
-		ropt->superuser = PQuser(AH->connection);
+		ropt->superuser = NULL;
 		ropt->suppressDumpWarnings = true;
 
 		savVerbose = AH->public.verbose;
@@ -848,14 +829,13 @@ _CloseArchive(ArchiveHandle *AH)
 				die_horribly(AH, modulename,
 				   "could not write null block at end of tar archive\n");
 		}
-
 	}
 
 	AH->FH = NULL;
 }
 
-static int
-_scriptOut(ArchiveHandle *AH, const void *buf, int len)
+static size_t
+_scriptOut(ArchiveHandle *AH, const void *buf, size_t len)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 
@@ -884,7 +864,6 @@ _StartBlobs(ArchiveHandle *AH, TocEntry *te)
 
 	sprintf(fname, "blobs.toc");
 	ctx->blobToc = tarOpen(AH, fname, 'w');
-
 }
 
 /*
@@ -910,12 +889,11 @@ _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 	else
 		sfx = "";
 
-	sprintf(fname, "blob_%d.dat%s", oid, sfx);
+	sprintf(fname, "blob_%u.dat%s", oid, sfx);
 
-	tarPrintf(AH, ctx->blobToc, "%d %s\n", oid, fname);
+	tarPrintf(AH, ctx->blobToc, "%u %s\n", oid, fname);
 
 	tctx->TH = tarOpen(AH, fname, 'w');
-
 }
 
 /*
@@ -947,7 +925,6 @@ _EndBlobs(ArchiveHandle *AH, TocEntry *te)
 	/* WriteInt(AH, 0); */
 
 	tarClose(AH, ctx->blobToc);
-
 }
 
 
@@ -962,7 +939,7 @@ tarPrintf(ArchiveHandle *AH, TAR_MEMBER *th, const char *fmt,...)
 {
 	char	   *p = NULL;
 	va_list		ap;
-	int			bSize = strlen(fmt) + 256;		/* Should be enough */
+	size_t		bSize = strlen(fmt) + 256;		/* Should be enough */
 	int			cnt = -1;
 
 	/*
@@ -1004,7 +981,7 @@ _tarChecksum(char *header)
 	return sum + 256;			/* Assume 8 blanks in checksum field */
 }
 
-int
+bool
 isValidTarHeader(char *header)
 {
 	int			sum;
@@ -1012,7 +989,17 @@ isValidTarHeader(char *header)
 
 	sscanf(&header[148], "%8o", &sum);
 
-	return (sum == chk && strncmp(&header[257], "ustar  ", 7) == 0);
+	if (sum != chk)
+		return false;
+
+	/* POSIX format */
+	if (strncmp(&header[257], "ustar00", 7) == 0)
+		return true;
+	/* older format */
+	if (strncmp(&header[257], "ustar  ", 7) == 0)
+		return true;
+
+	return false;
 }
 
 /* Given the member, write the TAR header & copy the file */
@@ -1022,18 +1009,20 @@ _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th)
 	lclContext *ctx = (lclContext *) AH->formatData;
 	FILE	   *tmp = th->tmpFH;	/* Grab it for convenience */
 	char		buf[32768];
-	int			cnt;
-	int			len = 0;
-	int			res;
-	int			i,
+	size_t		cnt;
+	off_t		len = 0;
+	size_t		res;
+	size_t		i,
 				pad;
 
 	/*
 	 * Find file len & go back to start.
 	 */
-	fseek(tmp, 0, SEEK_END);
-	th->fileLen = ftell(tmp);
-	fseek(tmp, 0, SEEK_SET);
+	fseeko(tmp, 0, SEEK_END);
+	th->fileLen = ftello(tmp);
+	if (th->fileLen > MAX_TAR_MEMBER_FILELEN)
+		die_horribly(AH, modulename, "archive member too large for tar format\n");
+	fseeko(tmp, 0, SEEK_SET);
 
 	_tarWriteHeader(th);
 
@@ -1041,7 +1030,9 @@ _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th)
 	{
 		res = fwrite(&buf[0], 1, cnt, th->tarFH);
 		if (res != cnt)
-			die_horribly(AH, modulename, "write error appending to tar archive (wrote %d, attempted %d)\n", res, cnt);
+			die_horribly(AH, modulename,
+						 "write error appending to tar archive (wrote %lu, attempted %lu)\n",
+						 (unsigned long) res, (unsigned long) cnt);
 		len += res;
 	}
 
@@ -1049,8 +1040,15 @@ _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th)
 		die_horribly(AH, modulename, "could not close tar member: %s\n", strerror(errno));
 
 	if (len != th->fileLen)
-		die_horribly(AH, modulename, "actual file length (%d) does not match expected (%d)\n",
-					 len, th->pos);
+	{
+		char		buf1[100],
+					buf2[100];
+
+		snprintf(buf1, 100, INT64_FORMAT, (int64) len);
+		snprintf(buf2, 100, INT64_FORMAT, (int64) th->pos);
+		die_horribly(AH, modulename, "actual file length (%s) does not match expected (%s)\n",
+					 buf1, buf2);
+	}
 
 	pad = ((len + 511) & ~511) - len;
 	for (i = 0; i < pad; i++)
@@ -1070,25 +1068,34 @@ _tarPositionTo(ArchiveHandle *AH, const char *filename)
 	TAR_MEMBER *th = calloc(1, sizeof(TAR_MEMBER));
 	char		c;
 	char		header[512];
-	int			i,
+	size_t		i,
 				len,
-				blks,
-				id;
+				blks;
+	int			id;
 
 	th->AH = AH;
 
 	/* Go to end of current file, if any */
 	if (ctx->tarFHpos != 0)
 	{
-		ahlog(AH, 4, "moving from position %d (%x) to next member at file position %d (%x)\n",
-			  ctx->tarFHpos, ctx->tarFHpos,
-			  ctx->tarNextMember, ctx->tarNextMember);
+		char		buf1[100],
+					buf2[100];
+
+		snprintf(buf1, 100, INT64_FORMAT, (int64) ctx->tarFHpos);
+		snprintf(buf2, 100, INT64_FORMAT, (int64) ctx->tarNextMember);
+		ahlog(AH, 4, "moving from position %s to next member at file position %s\n",
+			  buf1, buf2);
 
 		while (ctx->tarFHpos < ctx->tarNextMember)
 			_tarReadRaw(AH, &c, 1, NULL, ctx->tarFH);
 	}
 
-	ahlog(AH, 4, "now at file position %d (%x)\n", ctx->tarFHpos, ctx->tarFHpos);
+	{
+		char		buf[100];
+
+		snprintf(buf, 100, INT64_FORMAT, (int64) ctx->tarFHpos);
+		ahlog(AH, 4, "now at file position %s\n", buf);
+	}
 
 	/* We are at the start of the file. or at the next member */
 
@@ -1139,21 +1146,29 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 	char		h[512];
-	char		name[100];
+	char		tag[100];
 	int			sum,
 				chk;
-	int			len;
-	int			hPos;
+	size_t		len;
+	unsigned long	ullen;
+	off_t		hPos;
 	int			i;
 	bool		gotBlock = false;
 
 	while (!gotBlock)
 	{
 #if 0
-		if (ftell(ctx->tarFH) != ctx->tarFHpos)
+		if (ftello(ctx->tarFH) != ctx->tarFHpos)
+		{
+			char		buf1[100],
+						buf2[100];
+
+			snprintf(buf1, 100, INT64_FORMAT, (int64) ftello(ctx->tarFH));
+			snprintf(buf2, 100, INT64_FORMAT, (int64) ftello(ctx->tarFHpos));
 			die_horribly(AH, modulename,
-						 "mismatch in actual vs. predicted file position (%d vs. %d)\n",
-						 ftell(ctx->tarFH), ctx->tarFHpos);
+						 "mismatch in actual vs. predicted file position (%s vs. %s)\n",
+						 buf1, buf2);
+		}
 #endif
 
 		/* Save the pos for reporting purposes */
@@ -1165,7 +1180,9 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 			return 0;
 
 		if (len != 512)
-			die_horribly(AH, modulename, "incomplete tar header found (%d bytes)\n", len);
+			die_horribly(AH, modulename,
+						 "incomplete tar header found (%lu bytes)\n",
+						 (unsigned long) len);
 
 		/* Calc checksum */
 		chk = _tarChecksum(&h[0]);
@@ -1190,34 +1207,61 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 		}
 	}
 
-	sscanf(&h[0], "%99s", &name[0]);
-	sscanf(&h[124], "%12o", &len);
+	sscanf(&h[0], "%99s", tag);
+	sscanf(&h[124], "%12lo", &ullen);
+	len = (size_t) ullen;
 	sscanf(&h[148], "%8o", &sum);
 
-	ahlog(AH, 3, "TOC Entry %s at %d (length %d, checksum %d)\n", &name[0], hPos, len, sum);
+	{
+		char		buf[100];
+
+		snprintf(buf, 100, INT64_FORMAT, (int64) hPos);
+		ahlog(AH, 3, "TOC Entry %s at %s (length %lu, checksum %d)\n",
+			  tag, buf, (unsigned long) len, sum);
+	}
 
 	if (chk != sum)
+	{
+		char		buf[100];
+
+		snprintf(buf, 100, INT64_FORMAT, (int64) ftello(ctx->tarFH));
 		die_horribly(AH, modulename,
 					 "corrupt tar header found in %s "
-		"(expected %d (%o), computed %d (%o)) file position %ld (%lx)\n",
-					 &name[0], sum, sum, chk, chk, ftell(ctx->tarFH), ftell(ctx->tarFH));
+					 "(expected %d, computed %d) file position %s\n",
+					 tag, sum, chk, buf);
+	}
 
-	th->targetFile = strdup(name);
+	th->targetFile = strdup(tag);
 	th->fileLen = len;
 
 	return 1;
 }
 
+
+/*
+ * Utility routine to print possibly larger than 32 bit integers in a
+ * portable fashion.  Filled with zeros.
+ */
+static void print_val(char *s, uint64 val, unsigned int base, size_t len)
+{
+        int i;
+        for (i = len; i > 0; i--)
+        {
+                int digit = val % base;
+                s[i - 1] = '0' + digit;
+                val = val / base;
+        }
+}
+
+
 static void
 _tarWriteHeader(TAR_MEMBER *th)
 {
 	char		h[512];
-	int			i;
 	int			lastSum = 0;
 	int			sum;
 
-	for (i = 0; i < 512; i++)
-		h[i] = '\0';
+	memset(h, 0, sizeof(h));
 
 	/* Name 100 */
 	sprintf(&h[0], "%.99s", th->targetFile);
@@ -1226,33 +1270,30 @@ _tarWriteHeader(TAR_MEMBER *th)
 	sprintf(&h[100], "100600 ");
 
 	/* User ID 8 */
-	sprintf(&h[108], " 04000 ");
+	sprintf(&h[108], "004000 ");
 
 	/* Group 8 */
-	sprintf(&h[116], " 02000 ");
+	sprintf(&h[116], "002000 ");
 
-	/* File size 12 */
-	sprintf(&h[124], "%10o ", th->fileLen);
+	/* File size 12 - 11 digits, 1 space, no NUL */
+	print_val(&h[124], th->fileLen, 8, 11);
+	sprintf(&h[135], " ");
 
 	/* Mod Time 12 */
-	sprintf(&h[136], "%10o ", (int) time(NULL));
+	sprintf(&h[136], "%011o ", (int) time(NULL));
 
 	/* Checksum 8 */
-	sprintf(&h[148], "%6o ", lastSum);
+	sprintf(&h[148], "%06o ", lastSum);
 
-	/* Type 1 */
-	/* sprintf(&h[156], "%c", LF_NORMAL); */
+	/* Type - regular file */
 	sprintf(&h[156], "0");
 
-	/* Link name 100 (NULL) */
+	/* Link tag 100 (NULL) */
 
-	/* Magic 8 */
-	sprintf(&h[257], "ustar  ");
-
-	/*
-	 * GNU Version... sprintf(&h[257], "ustar"); sprintf(&h[263], "00");
-	 */
+	/* Magic 6 + Version 2 */
+	sprintf(&h[257], "ustar00");
 
+#if 0
 	/* User 32 */
 	sprintf(&h[265], "%.31s", "");		/* How do I get username reliably?
 										 * Do I need to? */
@@ -1262,15 +1303,15 @@ _tarWriteHeader(TAR_MEMBER *th)
 										 * I need to? */
 
 	/* Maj Dev 8 */
-	/* sprintf(&h[329], "%6o ", 0); */
-
-	/* Min Dev */
-	/* sprintf(&h[337], "%6o ", 0); */
+	sprintf(&h[329], "%6o ", 0);
 
+	/* Min Dev 8 */
+	sprintf(&h[337], "%6o ", 0);
+#endif
 
 	while ((sum = _tarChecksum(h)) != lastSum)
 	{
-		sprintf(&h[148], "%6o ", sum);
+		sprintf(&h[148], "%06o ", sum);
 		lastSum = sum;
 	}
 
